@@ -31,7 +31,6 @@ from ..utils.log import (
     log_to_stream,
     log_to_rotating_file,
     create_logger,
-    log_runtime_info,
 )
 from ..utils.error_handling import (
     exit_on_exception,
@@ -285,8 +284,8 @@ class Cli(Subcommands):
             overrides={
                 "files": dict(help="Path(s) to encrypted package(s)"),
                 "protocol": dict(
-                    help="The protocol for the file transfer."
-                    "Currently available: {}".format(", ".join(available_protocols)),
+                    help=f"The protocol for the file transfer."
+                    f"Currently available: {', '.join(available_protocols)}",
                     type=parse_protocol,
                     alias="-p",
                 ),
@@ -314,7 +313,10 @@ class Cli(Subcommands):
             overrides={
                 "files": dict(help="Path(s) to encrypted package(s)"),
                 "output_dir": dict(
-                    help="Output directory.", default=os.getcwd(), alias="-o"
+                    help="Path to the directory where package content is saved after decryption. "
+                    "If no path is specified, current working directory is taken.",
+                    default=os.getcwd(),
+                    alias="-o",
                 ),
                 "decrypt_only": dict(help="Skip extraction."),
                 "dry_run": dry_run_override,
@@ -358,7 +360,6 @@ log_to_stream()
 @when(Cli.config.error_reports, error_report_hint_at_exit(Cli.config))
 @log_exceptions(logger)
 def run():
-    log_runtime_info(logger)
     if Cli():
         return 0
     return 1
