@@ -179,15 +179,14 @@ def arguments_by_signature(
     """Builds argparser arguments according to the signature of a function f.
     Type hint annotations are used to infer the type for argparse"""
     sig = inspect.signature(f)
+
     if overrides is None:
         overrides = {}
     unknown_parameters = set(overrides) - set(sig.parameters)
     if unknown_parameters:
         raise ValueError(
-            "Function {} does not have arguments {} "
-            "(which have been provided as parameter docs)".format(
-                f.__name__, ", ".join(unknown_parameters)
-            )
+            f"Function {f.__name__} does not have arguments {', '.join(unknown_parameters)} "
+            f"(which have been provided as parameter docs)"
         )
     has_keyword_only = any(
         p.kind is inspect.Parameter.KEYWORD_ONLY for p in sig.parameters.values()
@@ -206,7 +205,7 @@ def arguments_by_signature(
         args = dict(help=p_overrides.get("help"))
         t = p_overrides.get("type", p.annotation)
         if t is inspect.Signature.empty:
-            raise ValueError(f"Argument {arg_names[0]} has no type annotation")
+            raise ValueError(f"Argument {arg_names[0]} has no type hint")
         try:
             t = unwrap_opt(t, default)
         except ValueError as e:
