@@ -1,12 +1,12 @@
-import os
 import io
+import os
 import tarfile
 import time
 from abc import ABC, abstractmethod
-from pathlib import Path
 from contextlib import contextmanager
 from itertools import filterfalse
-from typing import List, Iterable, IO, Sequence, Union
+from pathlib import Path
+from typing import List, Iterable, IO, Sequence, Union, Optional
 
 try:
     from typing import TypedDict
@@ -19,7 +19,6 @@ from .error import UserError
 from .filesystem import delete_files
 from ..utils.progress import ProgressInterface, with_progress, progress_iter
 
-
 tarfile.DEFAULT_FORMAT = tarfile.PAX_FORMAT
 
 METADATA_FILE = "metadata.json"
@@ -27,8 +26,18 @@ METADATA_FILE_SIG = "metadata.json.sig"
 CHECKSUM_FILE = "checksum.sha256"
 DATA_ARCHIVE = "data.tar.gz"
 DATA_FILE_ENCRYPTED = DATA_ARCHIVE + ".gpg"
-PACKAGE_CONTENT = {METADATA_FILE, METADATA_FILE_SIG, DATA_FILE_ENCRYPTED}
+PACKAGE_CONTENT = {
+    METADATA_FILE: "'{}' file with metadata",
+    METADATA_FILE_SIG: "'{}' file with a detached signature of the metadata",
+    DATA_FILE_ENCRYPTED: "'{}' file with the encrypted data",
+}
 CONTENT_FOLDER = "content"
+EXACTNESS_MSG = (
+    "BioMedIT input archive files MUST EXACTLY contain following files:\n"
+    + "\n".join(
+        ["  - " + template.format(entry) for entry, template in PACKAGE_CONTENT.items()]
+    )
+)
 
 
 def check_package(path: str) -> None:
@@ -46,17 +55,26 @@ def check_package(path: str) -> None:
             content = archive.namelist()
     except TypeError as e:
         raise UserError(e) from e
+    if not content:
+        raise UserError(f"Input archive '{path}' is empty.")
     assert_relative(content)
-    if set(content) != PACKAGE_CONTENT:
+    diff = PACKAGE_CONTENT.keys() - set(content)
+    if diff:
         raise UserError(
-            f"Input file '{path}' has non-compliant content:\n"
-            + "\n".join(content)
-            + "\n\n"
-            "BioMedIT input tar files must contain exactly the following "
-            "files:\n"
-            f"\t- {METADATA_FILE} file with metadata\n"
-            f"\t- {METADATA_FILE_SIG} file with a detached signature of the metadata\n"
-            f"\t- {DATA_FILE_ENCRYPTED} file with the encrypted data"
+            f"Input file '{path}' has non-compliant content.\n"
+            "Following files are missing:\n  - '"
+            + "'\n  - '".join(diff)
+            + "'\n\n"
+            + EXACTNESS_MSG
+        )
+    diff = set(content) - PACKAGE_CONTENT.keys()
+    if diff:
+        raise UserError(
+            f"Input archive '{path}' has non-compliant content.\n"
+            "Following files are NOT part of the specification and are rejected:\n  - '"
+            + "'\n  - '".join(diff)
+            + "'\n\n"
+            + EXACTNESS_MSG
         )
 
 
@@ -109,6 +127,7 @@ def write_tar(
         compression) to 9 (highest compression).
     :raises UserError:
     """
+
     # Create a TypedDict class so that the mypy linter does not complain
     # about "Incompatible types in assignment".
     class TarfileArguments(TypedDict, total=False):
@@ -190,7 +209,9 @@ def check_extracted(paths: Iterable[str]):
 
 
 @contextmanager
-def extract_with_progress(path: str, progress: ProgressInterface, *files: str):
+def extract_with_progress(
+    path: str, progress: Optional[ProgressInterface], *files: str
+):
     """Extracts given files from given archive.
 
     If only one file is specified, then progression scales with extracting of
