@@ -1,7 +1,6 @@
 import enum
 import io
 import re
-import urllib
 import warnings
 from typing import Iterable, List, Optional, Tuple, Union, Type, IO, Callable, cast
 from functools import wraps, partial
@@ -13,7 +12,6 @@ from .error import UserError
 from .request import urlopen
 from .secret import Secret
 from ..utils.config import Config
-from .. import APP_NAME_SHORT
 
 ExceptionType = Union[Type[BaseException], Tuple[Type[BaseException], ...]]
 
@@ -51,11 +49,8 @@ def to_user_error_gen(error_types: ExceptionType):
 
 
 validate_pub_key = to_user_error(RuntimeError)(crypt.validate_pub_key)
-retrieve_refresh_and_validate_keys = to_user_error_gen(
-    (RuntimeError, gpg.cmd.GPGError)
-)(partial(crypt.retrieve_refresh_and_validate_keys, url_opener=urlopen))
-refresh_keys = to_user_error((RuntimeError, gpg.cmd.GPGError))(
-    partial(crypt.refresh_keys, url_opener=urlopen)
+retrieve_refresh_and_validate_keys = to_user_error_gen((RuntimeError, gpg.GPGError))(
+    partial(crypt.retrieve_refresh_and_validate_keys, url_opener=urlopen)
 )
 fingerprint2keyid = to_user_error(RuntimeError)(crypt.fingerprint2keyid)
 verify_metadata_signature = to_user_error(RuntimeError)(
@@ -63,28 +58,25 @@ verify_metadata_signature = to_user_error(RuntimeError)(
 )
 
 
-@to_user_error(gpg.cmd.GPGError)
+@to_user_error((gpg.GPGError, gpg.KeyserverError))
 def search_keyserver(search_term: str, keyserver: str) -> List[gpg.KeyInfo]:
     """Search keyserver for keys matching the query"""
-    try:
-        return list(gpg.search_keyserver(search_term, keyserver, url_opener=urlopen))
-    except urllib.error.URLError:
-        raise KeyserverError(action="search", keyserver=keyserver) from None
+    return list(gpg.search_keyserver(search_term, keyserver, url_opener=urlopen))
 
 
-@to_user_error(gpg.cmd.GPGError)
+@to_user_error(gpg.GPGError)
 def import_keys(key_data: str, gpg_store: GPGStore):
     """Import keys from text"""
     return gpg_store.import_file(key_data.encode())
 
 
-@to_user_error(gpg.cmd.GPGError)
+@to_user_error(gpg.GPGError)
 def delete_pub_keys(fingerprints: List[str], gpg_store: GPGStore):
     """Delete public key"""
     return gpg_store.delete_pub_keys(*fingerprints)
 
 
-@to_user_error(gpg.cmd.GPGError)
+@to_user_error(gpg.GPGError)
 def create_revocation_certificate(
     fingerprint: str, passphrase: str, gpg_store: GPGStore
 ) -> bytes:
@@ -100,20 +92,16 @@ def verify_key_length(key: gpg.Key, min_key_length: int = 4096):
         )
 
 
+@to_user_error(gpg.KeyserverError)
 def upload_keys(fingerprints: List[str], keyserver: str, gpg_store: GPGStore):
     """Upload public key to keyserver"""
-    try:
-        gpg_store.send_keys(*fingerprints, keyserver=keyserver, url_opener=urlopen)
-    except urllib.error.URLError:
-        raise KeyserverError(action="upload", keyserver=keyserver) from None
+    gpg_store.send_keys(*fingerprints, keyserver=keyserver, url_opener=urlopen)
 
 
+@to_user_error((gpg.KeyserverError, gpg.KeyserverKeyNotFoundError))
 def download_keys(fingerprints: List[str], keyserver: str, gpg_store: GPGStore):
     """Download public key to keyserver"""
-    try:
-        gpg_store.recv_keys(*fingerprints, keyserver=keyserver, url_opener=urlopen)
-    except urllib.error.URLError:
-        raise KeyserverError(action="download", keyserver=keyserver) from None
+    gpg_store.recv_keys(*fingerprints, keyserver=keyserver, url_opener=urlopen)
 
 
 def detach_sign_file(
@@ -126,7 +114,7 @@ def detach_sign_file(
     try:
         with gpg_store.detach_sig(src, passphrase, signature_fingerprint) as out:
             return out.read()
-    except gpg.cmd.GPGError as e:
+    except gpg.GPGError as e:
         raise UserError(f"File signing failed. {e}") from e
 
 
@@ -149,7 +137,7 @@ def check_password(password: Secret, key_fingerprint: str, gpg_store: GPGStore):
             src=b"test", passphrase=password, signee=key_fingerprint
         ):
             pass
-    except gpg.cmd.GPGError:
+    except gpg.GPGError:
         raise UserError("GPG password is incorrect") from None
 
 
@@ -281,7 +269,7 @@ def encrypt_and_sign(
             sign=signature_fingerprint,
             compress_algo=gpg.CompressAlgo.NONE,
         )
-    except gpg.cmd.GPGError:
+    except gpg.GPGError:
         raise UserError(
             "Encryption failed. Maybe you entered a wrong passphrase."
         ) from None
@@ -331,7 +319,7 @@ def decrypt(
             sender_fprs = gpg_store.decrypt(
                 source=source, output=output, passphrase=passphrase
             )
-    except gpg.cmd.GPGError as e:
+    except gpg.GPGError as e:
         raise UserError(
             "Failed to decrypt. Error message from gpg:\n" + format(e)
         ) from e
@@ -382,18 +370,19 @@ def create_key(
 def load_authority_key(sett_config: Config) -> Optional[gpg.Key]:
     """Load (download if needed) and refresh the key authority's PGP key
 
-    The function returns None if any of the following:
+    The function returns None if either:
      * No key authority fingerprint is given in the config.
-     * Offline mode is on.
-     * The key matching the fingerprint could not be retrieved.
+     * The key matching the fingerprint could not be retrieved, neither from
+       the local keyring nor from the keyserver.
     """
-    if sett_config.key_authority_fingerprint and not sett_config.offline:
+    if sett_config.key_authority_fingerprint:
         try:
             return crypt.retrieve_and_refresh_certification_authority_key(
                 key_fingerprint=sett_config.key_authority_fingerprint,
                 gpg_store=sett_config.gpg_store,
                 keyserver_url=sett_config.keyserver_url,
                 validate_key_origin=False,
+                allow_key_download=sett_config.allow_gpg_key_autodownload,
                 url_opener=urlopen,
             )
         except RuntimeError as e:
@@ -401,28 +390,11 @@ def load_authority_key(sett_config: Config) -> Optional[gpg.Key]:
                 "The key validation authority's PGP key "
                 f"[{sett_config.key_authority_fingerprint}] could not be "
                 "found in your local keyring, nor could it be downloaded.\n"
-                "Please verify that the key fingerprint and the keyserver URL "
-                "are correct and that you are not in offline mode.\n"
+                "Please verify that the key validation authority fingerprint "
+                "and the keyserver URL settings are correct. Also verify that "
+                "you are not in offline mode and that PGP key auto-download "
+                "is enabled.\n"
                 f"Reason for failure: {e}"
             )
 
     return None
-
-
-class KeyserverError(UserError):
-    """Error class that displays an error message when the keyserver cannot
-    be reached.
-
-    :param action: either "download" or "upload".
-    :param keyserver: URL of keyserver.
-    """
-
-    def __init__(self, action: str, keyserver: str):
-        super().__init__(
-            f"Key {action} failed as {APP_NAME_SHORT} could not connect to the "
-            "keyserver.\n"
-            "A possible cause for this is that the keyserver is temporarily "
-            "unavailable.\n"
-            f"Please try to connect to [{keyserver}] using your web browser to "
-            "confirm whether the keyserver is available or not.\n"
-        )
