@@ -28,14 +28,14 @@ class PortalApi:
     def verify_transfer(self, metadata: MetaData, filename: str) -> str:
         """Verify transfer_id using external API.
 
-        Return project_id for a valid transfer_id.
+        Return project_code for a valid transfer_id.
         """
         data = json.dumps(
             {"file_name": filename, "metadata": json.dumps(MetaData.asdict(metadata))}
         ).encode()
         response = handleErrors(post)(self.base_url + self.dpkg_check, data)
         try:
-            return json.loads(response)["project_id"]
+            return json.loads(response)["project_code"]
         except json.decoder.JSONDecodeError as e:
             raise UserError(
                 "PortalApi.verify_transfer: got invalid json from portal: "
@@ -44,7 +44,7 @@ class PortalApi:
         except KeyError as e:
             raise UserError(
                 "PortalApi.verify_transfer: json response from portal does "
-                "not include field `project_id`: " + response.decode()
+                "not include field `project_code`: " + response.decode()
             ) from e
 
     def request_key_signature(self, keyid: str) -> bool:
