@@ -1,4 +1,4 @@
-from typing import Tuple, Callable, Type, Union, TypeVar
+from typing import Tuple, Callable, Union, TypeVar
 import inspect
 from dataclasses import is_dataclass, fields
 
@@ -31,10 +31,10 @@ def enforce_secret_by_signature(
 T = TypeVar("T")
 
 
-def enforce_secret(val: T, t: Type) -> Union[T, Secret]:
+def enforce_secret(val: T, t: type) -> Union[T, Secret]:
     if t is Secret or (  # check for Optional / Union:
         getattr(t, "__origin__", None) is Union
-        and Secret in t.__args__
+        and Secret in getattr(t, "__args__", ())
         and isinstance(val, str)
     ):
         return Secret(val)
