@@ -1,6 +1,7 @@
 import warnings
 import re
 import urllib.error
+from typing import Optional
 
 from sett import URL_READTHEDOCS, APP_NAME_SHORT
 from ..core.request import urlopen
@@ -35,7 +36,7 @@ def check_version(repo_url: str, gui_formatting: bool = False):
         )
 
 
-def get_latest_version(repo_url: str):
+def get_latest_version(repo_url: str) -> Optional[str]:
     try:
         url = repo_url + "/simple/" + __project_name__ + "/"
         with urlopen(url) as response:
@@ -49,9 +50,11 @@ def get_latest_version(repo_url: str):
             "Could not connect to pypi repository to query the latest version. "
             "You might have an outdated version. Please check yourself!"
         )
+        return None
     except IndexError:
         warnings.warn(
             "No releases found on the pypi repository! "
             "Please contact the developers!"
         )
+        return None
     return ".".join(map(str, version))
