@@ -1,8 +1,14 @@
 from functools import partial
 
-from .component import TabMixin, PathInput, GuiProgress, get_text_input
+from .component import (
+    TabMixin,
+    PathInput,
+    GuiProgress,
+    get_text_input,
+    vbox_layout,
+    hbox_layout,
+)
 from .file_selection_widget import ArchiveOnlyFileSelectionWidget
-from .parallel import Worker
 from .pyside import QtCore, QtWidgets
 from ..workflows import decrypt
 
@@ -10,7 +16,6 @@ from ..workflows import decrypt
 class DecryptTab(QtWidgets.QWidget, TabMixin):
     def __init__(self, *args, **kwargs):
         super().__init__(*args, **kwargs)
-        self.threadpool = QtCore.QThreadPool.globalInstance()
         self.app_data = self.parent().app_data
 
         files_panel = self.create_files_panel()
@@ -20,20 +25,20 @@ class DecryptTab(QtWidgets.QWidget, TabMixin):
         self.create_console()
         self.create_progress_bar()
 
-        layout = QtWidgets.QVBoxLayout(self)
-        layout.addWidget(files_panel)
-        layout.addWidget(decrypt_options_panel)
-        layout.addWidget(self.run_panel)
-        layout.addWidget(self.console)
-        layout.addWidget(self.progress_bar)
+        vbox_layout(
+            files_panel,
+            decrypt_options_panel,
+            self.run_panel,
+            self.console,
+            self.progress_bar,
+            parent=self,
+        )
 
     def _enable_buttons(self):
         self.set_buttons_enabled(len(self.app_data.decrypt_files) > 0)
 
     def create_files_panel(self):
-        box = ArchiveOnlyFileSelectionWidget(
-            "Files to decrypt (hint: you can drag & drop files)", self
-        )
+        box = ArchiveOnlyFileSelectionWidget(title="Files to decrypt", parent=self)
         box.file_list_model.layoutChanged.connect(
             lambda: setattr(self.app_data, "decrypt_files", box.get_list())
         )
@@ -57,36 +62,31 @@ class DecryptTab(QtWidgets.QWidget, TabMixin):
             partial(setattr, self.app_data, "decrypt_output_location")
         )
 
-        layout_output = QtWidgets.QHBoxLayout()
-        layout_output.addWidget(QtWidgets.QLabel("Location"))
-        layout_output.addWidget(output_location.text)
-        layout_output.addWidget(output_location.btn)
-        layout = QtWidgets.QVBoxLayout(box)
-        layout.addWidget(decompress)
-        layout.addLayout(layout_output)
+        layout_output = hbox_layout(
+            QtWidgets.QLabel("Location"), output_location.text, output_location.btn
+        )
+        vbox_layout(decompress, layout_output, parent=box)
         return box
 
     def decrypt(self, dry_run=False):
-        progress = GuiProgress()
-        progress.updated.connect(self.progress_bar.setValue)
         if not dry_run:
             pw = get_text_input(self, "Enter password for your GPG key")
             if pw is None:
                 return
         else:
             pw = None
-        worker = Worker(
+        self.run_workflow_thread(
             decrypt.decrypt,
-            files=self.app_data.decrypt_files,
+            f_kwargs=dict(
+                files=self.app_data.decrypt_files,
+                output_dir=str(self.app_data.decrypt_output_location),
+                config=self.app_data.config,
+                decrypt_only=self.app_data.decrypt_decrypt_only,
+                passphrase=pw,
+                dry_run=dry_run,
+                progress=GuiProgress(self.progress_bar.setValue),
+            ),
             capture_loggers=(decrypt.logger,),
-            output_dir=str(self.app_data.decrypt_output_location),
-            config=self.app_data.config,
-            decrypt_only=self.app_data.decrypt_decrypt_only,
-            passphrase=pw,
-            dry_run=dry_run,
-            progress=progress,
             ignore_exceptions=True,
             report_config=self.app_data.config,
         )
-        self.add_worker_actions(worker)
-        self.threadpool.start(worker)
