@@ -10,12 +10,15 @@ from .component import (
     MandatoryLabel,
     create_verify_dtr_checkbox,
     LineEdit,
+    grid_layout,
+    GridLayoutCell,
+    vbox_layout,
+    hbox_layout,
 )
 from .file_selection_widget import DirectoryAndFileSelectionWidget
 from .model import TableModel
-from .parallel import Worker
 from .pyside import QtCore, QtGui, QtWidgets
-from .settings_tab import LABEL_COMPRESSION_LEVEL, STATUS_TIP_COMPRESSION_LEVEL
+from ..utils.config import LABEL_COMPRESSION_LEVEL, DESCRIPTION_COMPRESSION_LEVEL
 from ..core.metadata import Purpose
 from ..core.filesystem import OutOfSpaceError
 from ..utils.validation import PACKAGE_SUFFIX
@@ -40,7 +43,6 @@ class EncryptTab(QtWidgets.QWidget, TabMixin):
     def __init__(self, *args, **kwargs):
         super().__init__(*args, **kwargs)
 
-        self.threadpool = QtCore.QThreadPool.globalInstance()
         self.app_data = self.parent().app_data
 
         files_panel = self.create_files_panel()
@@ -52,12 +54,14 @@ class EncryptTab(QtWidgets.QWidget, TabMixin):
         self.create_console()
         self.create_progress_bar()
 
-        layout = QtWidgets.QVBoxLayout(self)
-        layout.addWidget(files_panel)
-        layout.addLayout(self._create_middle_layout())
-        layout.addWidget(self.run_panel)
-        layout.addWidget(self.console)
-        layout.addWidget(self.progress_bar)
+        vbox_layout(
+            files_panel,
+            self._create_sender_transferinfo_and_output_layout(),
+            self.run_panel,
+            self.console,
+            self.progress_bar,
+            parent=self,
+        )
 
     def _enable_buttons(self):
         self.set_buttons_enabled(
@@ -66,17 +70,14 @@ class EncryptTab(QtWidgets.QWidget, TabMixin):
         )
 
     def create_files_panel(self):
-        box = DirectoryAndFileSelectionWidget(
-            "Files and/or directories to encrypt (hint: you can drag & drop files)",
-            self,
-        )
+        box = DirectoryAndFileSelectionWidget(title="Data to encrypt", parent=self)
         box.file_list_model.layoutChanged.connect(
             lambda: setattr(self.app_data, "encrypt_files", box.get_list())
         )
         return box
 
     def create_user_panel(self):
-        group_box = QtWidgets.QGroupBox("Keys")
+        group_box = QtWidgets.QGroupBox("Data sender and recipients")
         sender_widget = QtWidgets.QComboBox()
         sender_widget.setModel(self.app_data.priv_keys_model)
         if self.app_data.default_key_index:
@@ -141,36 +142,37 @@ class EncryptTab(QtWidgets.QWidget, TabMixin):
         # Set the default value for the sender
         update_sender(sender_widget.currentIndex())
 
-        layout = QtWidgets.QGridLayout(group_box)
-        layout.addWidget(
-            QtWidgets.QLabel("Select data sender and add at least one recipient"),
-            0,
-            0,
-            1,
-            3,
+        grid_layout(
+            (
+                GridLayoutCell(
+                    QtWidgets.QLabel(
+                        "Select data sender and add at least one recipient:"
+                    ),
+                    span=3,
+                ),
+            ),
+            (MandatoryLabel("Sender"), sender_widget),
+            (None, recipients_input_view, recipients_btn_add),
+            (
+                GridLayoutCell(MandatoryLabel("Recipients"), align=QtCore.Qt.AlignTop),
+                recipients_output_view,
+                GridLayoutCell(recipients_btn_remove, align=QtCore.Qt.AlignTop),
+            ),
+            parent=group_box,
         )
-        layout.addWidget(MandatoryLabel("Sender"), 1, 0, 1, 1)
-        layout.addWidget(sender_widget, 1, 1, 1, 1)
-        layout.addWidget(recipients_input_view, 2, 1, 1, 1)
-        layout.addWidget(recipients_btn_add, 2, 2, 1, 1)
-        layout.addWidget(MandatoryLabel("Recipients"), 3, 0, 1, 1, QtCore.Qt.AlignTop)
-        layout.addWidget(recipients_output_view, 3, 1, 1, 1)
-        layout.addWidget(
-            recipients_btn_remove, 3, 2, 1, 1, alignment=QtCore.Qt.AlignTop
-        )
-
         return group_box
 
-    def _create_middle_layout(self):
-        layout_right = QtWidgets.QVBoxLayout()
-        layout_right.addWidget(self._create_metadata_group_box())
-        layout_right.addWidget(self._create_output_group_box())
-        layout = QtWidgets.QHBoxLayout()
-        layout.addWidget(self.create_user_panel())
-        layout.addLayout(layout_right)
-        return layout
+    def _create_sender_transferinfo_and_output_layout(self):
+        """Create a layout with 3 group boxes: 'Data sender and recipients',
+        'Data transfer info' and 'Output'.
+        """
+        layout_right = vbox_layout(
+            self._create_transferinfo_group_box(), self._create_output_group_box()
+        )
+        return hbox_layout(self.create_user_panel(), layout_right)
 
     def _create_output_group_box(self):
+        """Create a group box with the output-related fields."""
         group_box = QtWidgets.QGroupBox("Output")
         # Create fields
         output_suffix = LineEdit()
@@ -197,12 +199,12 @@ class EncryptTab(QtWidgets.QWidget, TabMixin):
             minimum=0,
             maximum=9,
             initial_value=self.app_data.encrypt_compression_level,
-            status_tip=STATUS_TIP_COMPRESSION_LEVEL,
+            status_tip=DESCRIPTION_COMPRESSION_LEVEL,
             on_change=lambda n: setattr(self.app_data, "encrypt_compression_level", n),
         )
-        compression_layout = QtWidgets.QHBoxLayout()
-        for w in (QtWidgets.QLabel(LABEL_COMPRESSION_LEVEL),) + slider_widgets:
-            compression_layout.addWidget(w)
+        compression_layout = hbox_layout(
+            *(QtWidgets.QLabel(LABEL_COMPRESSION_LEVEL),) + slider_widgets
+        )
 
         ignore_disk_space_error = QtWidgets.QCheckBox(
             ignore_disk_space_error_label, group_box
@@ -221,18 +223,22 @@ class EncryptTab(QtWidgets.QWidget, TabMixin):
             )
         )
 
-        layout = QtWidgets.QGridLayout(group_box)
-        layout.addWidget(QtWidgets.QLabel("Suffix"), 0, 0, 1, 1)
-        layout.addWidget(output_suffix, 0, 1, 1, 1)
-        layout.addWidget(QtWidgets.QLabel("Location"), 1, 0, 1, 1)
-        layout.addWidget(output_location.text, 1, 1, 1, 1)
-        layout.addWidget(output_location.btn, 1, 2, 1, 1)
-        layout.addLayout(compression_layout, 2, 0, 1, 2)
-        layout.addWidget(ignore_disk_space_error, 3, 0, 1, 2)
+        grid_layout(
+            (QtWidgets.QLabel("Suffix"), output_suffix),
+            (
+                QtWidgets.QLabel("Location"),
+                output_location.text,
+                output_location.btn,
+            ),
+            (GridLayoutCell(compression_layout, span=2),),
+            (GridLayoutCell(ignore_disk_space_error, span=2),),
+            parent=group_box,
+        )
         return group_box
 
-    def _create_metadata_group_box(self):
-        group_box = QtWidgets.QGroupBox("Metadata")
+    def _create_transferinfo_group_box(self):
+        """Create a group box with data transfer request (DTR) related fields."""
+        group_box = QtWidgets.QGroupBox("Data transfer info")
         # Create fields
         transfer_id = LineEdit()
         transfer_id.setStatusTip("(optional) Data Transfer Request ID")
@@ -255,42 +261,40 @@ class EncryptTab(QtWidgets.QWidget, TabMixin):
                 self.app_data, "encrypt_purpose", None if text == "" else Purpose(text)
             )
         )
-        layout = QtWidgets.QGridLayout(group_box)
-        layout.addWidget(QtWidgets.QLabel("DTR ID"), 0, 0, 1, 1)
-        layout.addWidget(transfer_id, 0, 1, 1, 1)
-        layout.addWidget(verify_dtr, 1, 0, 1, 2)
-        layout.addWidget(QtWidgets.QLabel("Purpose"), 2, 0, 1, 1)
-        layout.addWidget(purpose, 2, 1, 1, 1)
+        grid_layout(
+            (QtWidgets.QLabel("DTR ID"), transfer_id),
+            (GridLayoutCell(verify_dtr, span=2),),
+            (QtWidgets.QLabel("Purpose"), purpose),
+            parent=group_box,
+        )
         return group_box
 
     def encrypt(self, dry_run=False):
-        progress = GuiProgress()
-        progress.updated.connect(self.progress_bar.setValue)
         if not dry_run and self.app_data.config.sign_encrypted_data:
             pw = get_text_input(self, "Enter password for your GPG key")
             if pw is None:
                 return
         else:
             pw = None
-        worker = Worker(
+        self.run_workflow_thread(
             encrypt_workflow,
-            self.app_data.encrypt_files,
-            capture_loggers=(encrypt.logger,),
-            sender=self.app_data.encrypt_sender,
-            recipient=self.app_data.encrypt_recipients,
-            dtr_id=self.app_data.encrypt_transfer_id,
-            verify_dtr=self.app_data.encrypt_verify_dtr,
-            config=self.app_data.config,
-            passphrase=pw,
-            output=self.app_data.encrypt_output_location,
-            output_suffix=self.app_data.encrypt_output_suffix,
-            dry_run=dry_run,
-            compression_level=self.app_data.encrypt_compression_level,
-            purpose=self.app_data.encrypt_purpose,
-            progress=progress,
+            f_kwargs=dict(
+                files=self.app_data.encrypt_files,
+                capture_loggers=(encrypt.logger,),
+                sender=self.app_data.encrypt_sender,
+                recipient=self.app_data.encrypt_recipients,
+                dtr_id=self.app_data.encrypt_transfer_id,
+                verify_dtr=self.app_data.encrypt_verify_dtr,
+                config=self.app_data.config,
+                passphrase=pw,
+                output=self.app_data.encrypt_output_location,
+                output_suffix=self.app_data.encrypt_output_suffix,
+                dry_run=dry_run,
+                compression_level=self.app_data.encrypt_compression_level,
+                purpose=self.app_data.encrypt_purpose,
+                progress=GuiProgress(self.progress_bar.setValue),
+            ),
             ignore_exceptions=True,
             report_config=self.app_data.config,
             force=self.app_data.encrypt_ignore_disk_space_error,
         )
-        self.add_worker_actions(worker)
-        self.threadpool.start(worker)
