@@ -7,7 +7,7 @@ from .component import (
     MandatoryLabel,
     show_warning,
     ToolBar,
-    Action,
+    vbox_layout,
 )
 from .pyside import QtWidgets, QtCore, QtGui, QAction, open_window
 
@@ -15,7 +15,7 @@ from .pyside import QtWidgets, QtCore, QtGui, QAction, open_window
 class FileSelectionWidget(QtWidgets.QGroupBox):
     """File selection widget"""
 
-    def __init__(self, label_text, parent, name_filter=None):
+    def __init__(self, title, parent, name_filter=None):
         super().__init__(parent)
         self.path = str(Path.home())
         self.name_filter = name_filter
@@ -28,15 +28,23 @@ class FileSelectionWidget(QtWidgets.QGroupBox):
         )
         self.file_list_view.setLayout(QtWidgets.QVBoxLayout())
 
-        toolbar = ToolBar("Add files", self)
+        self.setTitle(title)
+        self._create_layout()
+        self.setAcceptDrops(True)
+
+    def _create_layout(self, allow_dirs=False):
+        input_type_label = "files" + (" and/or directories" if allow_dirs else "")
+        toolbar = ToolBar(f"Add {input_type_label}", self)
         for action in self._create_actions():
             toolbar.addAction(action)
-        layout = QtWidgets.QVBoxLayout(self)
-        layout.addWidget(MandatoryLabel(label_text))
-        layout.addWidget(toolbar)
-        layout.addWidget(self.file_list_view)
-
-        self.setAcceptDrops(True)
+        vbox_layout(
+            MandatoryLabel(
+                f"Select one or more {input_type_label} (drag & drop is supported):"
+            ),
+            toolbar,
+            self.file_list_view,
+            parent=self,
+        )
 
     # pylint: disable=no-self-use
     def dragEnterEvent(self, event: QtGui.QDragEnterEvent) -> None:
@@ -60,7 +68,7 @@ class FileSelectionWidget(QtWidgets.QGroupBox):
         ]
 
     def _create_add_files_action(self) -> QAction:
-        action = Action(QtGui.QIcon(":icon/feather/file-plus.png"), "Add file", self)
+        action = QAction(QtGui.QIcon(":icon/feather/file-plus.png"), "Add file", self)
         action.triggered.connect(self._add_files)
         return action
 
@@ -87,7 +95,7 @@ class FileSelectionWidget(QtWidgets.QGroupBox):
             self.file_list_model.setStringList([])
             self.file_list_model.layoutChanged.emit()
 
-        action = Action(QtGui.QIcon(":icon/feather/trash-2.png"), "Clear list", self)
+        action = QAction(QtGui.QIcon(":icon/feather/trash-2.png"), "Clear list", self)
         action.triggered.connect(clear_list)
         return action
 
@@ -114,8 +122,11 @@ class FileSelectionWidget(QtWidgets.QGroupBox):
 class DirectoryAndFileSelectionWidget(FileSelectionWidget):
     """File selection widget extension adding directory selection"""
 
-    def __init__(self, label_text, parent):
-        super().__init__(label_text, parent)
+    def __init__(self, title, parent):
+        super().__init__(title, parent)
+
+    def _create_layout(self, allow_dirs=True):
+        super()._create_layout(allow_dirs=allow_dirs)
 
     def _create_actions(self):
         actions = super()._create_actions()
@@ -129,7 +140,7 @@ class DirectoryAndFileSelectionWidget(FileSelectionWidget):
             )
             self._update_paths((directory,))
 
-        action = Action(
+        action = QAction(
             QtGui.QIcon(":icon/feather/folder-plus.png"), "Add directory", self
         )
         action.triggered.connect(add_directory)
@@ -139,8 +150,8 @@ class DirectoryAndFileSelectionWidget(FileSelectionWidget):
 class ArchiveOnlyFileSelectionWidget(FileSelectionWidget):
     """File selection widget extension for selecting ZIP and TAR archives only."""
 
-    def __init__(self, label_text, parent):
-        super().__init__(label_text, parent, "Archives (*.zip *.tar)")
+    def __init__(self, title, parent):
+        super().__init__(title, parent, name_filter="Archives (*.zip *.tar)")
 
     def dropEvent(self, event: QtGui.QDropEvent) -> None:
         paths = [
