@@ -1,8 +1,7 @@
 from typing import Optional
 
-from .pyside import QtCore
 from .model import AppData
-from .parallel import Worker
+from .parallel import run_thread
 from .component import warning_callback
 from ..core.crypt import gpg, load_authority_key
 
@@ -18,12 +17,9 @@ def load_authority_key_threaded(app_data: AppData) -> None:
         app_data.validation_authority_key = key
 
     show_warning = warning_callback("Certification Authority Key Loading Error")
-    worker = Worker(
+    run_thread(
         load_authority_key,
-        report_config=app_data.config,
-        sett_config=app_data.config,
+        f_kwargs=dict(report_config=app_data.config, sett_config=app_data.config),
         forward_errors=show_warning,
+        signals=dict(warning=show_warning, result=update_authority_key_in_app_data),
     )
-    worker.signals.warning.connect(show_warning)
-    worker.signals.result.connect(update_authority_key_in_app_data)
-    QtCore.QThreadPool.globalInstance().start(worker)
