@@ -3,10 +3,18 @@ from pathlib import Path
 from typing import Iterator, Optional, Sequence, Callable
 
 from sett.core.error import UserError
-from .component import SelectionAction, ToolBar, LineEdit, Action, warning_callback
-from .parallel import Worker
-from .pyside import QtCore, QtGui, QtWidgets, QAction, open_window
+from .component import (
+    SelectionAction,
+    SelectionButton,
+    ToolBar,
+    LineEdit,
+    warning_callback,
+    grid_layout,
+    GridLayoutCell,
+)
 from .keys_refresh import load_authority_key_threaded
+from .parallel import run_thread
+from .pyside import QtCore, QtGui, QtWidgets, QAction, open_window
 from ..core import crypt
 from ..core.crypt import gpg
 from ..utils.config import Config
@@ -19,7 +27,6 @@ from ..workflows import (
 class KeysTab(QtWidgets.QWidget):
     def __init__(self, *args, **kwargs):
         super().__init__(*args, **kwargs)
-        self.threadpool = QtCore.QThreadPool.globalInstance()
         self.app_data = self.parent().app_data
 
         # Update public and private key lists in AppData.
@@ -59,7 +66,7 @@ class KeysTab(QtWidgets.QWidget):
             lambda: self.priv_keys_view.selectionModel().clear()
         )
 
-        action_generate_key = Action(
+        action_generate_key = QAction(
             QtGui.QIcon(":icon/feather/plus-square.png"),
             "Generate new private/public key",
             self,
@@ -86,14 +93,14 @@ class KeysTab(QtWidgets.QWidget):
         toolbar.addSeparator()
         toolbar.addAction(action_refresh_keys)
 
-        layout = QtWidgets.QGridLayout()
-        layout.addWidget(toolbar, 0, 0, 1, 2)
-        layout.addWidget(QtWidgets.QLabel("Private keys"), 1, 0)
-        layout.addWidget(self.priv_keys_view, 2, 0)
-        layout.addWidget(QtWidgets.QLabel("Public keys"), 1, 1)
-        layout.addWidget(self.pub_keys_view, 2, 1)
-        layout.addWidget(self.text_panel, 3, 0, 1, 2)
-        self.setLayout(layout)
+        self.setLayout(
+            grid_layout(
+                (GridLayoutCell(toolbar, span=2),),
+                (QtWidgets.QLabel("Private keys"), QtWidgets.QLabel("Public keys")),
+                (self.priv_keys_view, self.pub_keys_view),
+                (GridLayoutCell(self.text_panel, span=2),),
+            )
+        )
 
     def key_to_html(self, key: gpg.Key) -> str:
         """Represent a PGP key as an HTML string"""
@@ -169,7 +176,7 @@ class KeysTab(QtWidgets.QWidget):
             """Force disable button in offline mode"""
             icon_obj = f":icon/feather/{icon}.png"
             if self.app_data.config.offline:
-                action = Action(
+                action = QAction(
                     QtGui.QIcon(icon_obj),
                     f"{tip} (not available in the offline mode)",
                     self,
@@ -184,7 +191,7 @@ class KeysTab(QtWidgets.QWidget):
                         selection_model=selection_model,
                     )
                 else:
-                    action = Action(QtGui.QIcon(icon_obj), tip, self)
+                    action = QAction(QtGui.QIcon(icon_obj), tip, self)
             return action
 
         for action, fn in (
@@ -218,7 +225,7 @@ class KeysTab(QtWidgets.QWidget):
                 self.delete_keys,
             ),
             (
-                Action(
+                QAction(
                     QtGui.QIcon(":icon/feather/file-plus.png"),
                     "Import key from file",
                     self,
@@ -266,22 +273,23 @@ class KeysTab(QtWidgets.QWidget):
             for index in self.pub_keys_view.selectedIndexes()
         ]
         if selected_keys:
-            worker = Worker(
+
+            def on_result():
+                self.update_public_keys()
+                self.update_display_selected_pub_key()
+                show_ok()
+
+            run_thread(
                 crypt.download_keys,
-                selected_keys,
-                self.app_data.config.keyserver_url,
-                self.app_data.config.gpg_store,
+                f_kwargs=dict(
+                    fingerprints=selected_keys,
+                    keyserver=self.app_data.config.keyserver_url,
+                    gpg_store=self.app_data.config.gpg_store,
+                ),
                 report_config=self.app_data.config,
                 forward_errors=show_warning,
+                signals=dict(result=on_result),
             )
-            worker.signals.result.connect(
-                lambda: (
-                    self.update_public_keys(),
-                    self.update_display_selected_pub_key(),
-                    show_ok(),
-                )
-            )
-            self.threadpool.start(worker)
 
     def import_key(self):
         """Import a GPG key from a local file"""
@@ -292,7 +300,7 @@ class KeysTab(QtWidgets.QWidget):
         msg.setWindowTitle("GPG public key import")
         try:
             if path:
-                with open(path) as fin:
+                with open(path, encoding="utf-8") as fin:
                     key_data = fin.read()
                 crypt.import_keys(key_data, self.app_data.config.gpg_store)
                 self.update_public_keys()
@@ -369,23 +377,23 @@ class KeysTab(QtWidgets.QWidget):
                 click_show_details(msgbox=msg)
             status = open_window(msg)
             if status == QtWidgets.QMessageBox.Ok:
-                worker = Worker(
+                run_thread(
                     upload_keys_workflow.upload_keys,
-                    [key.fingerprint],
-                    config=self.app_data.config,
+                    f_kwargs=dict(
+                        key_ids=[key.fingerprint], config=self.app_data.config
+                    ),
                     report_config=self.app_data.config,
                     capture_loggers=(upload_keys_workflow.logger,),
                     forward_errors=show_warning,
+                    signals=dict(result=lambda _: show_ok()),
                 )
-                worker.signals.result.connect(lambda _: show_ok())
-                self.threadpool.start(worker)
 
     def send_signature_request(self):
         selected_keys = [
             index.model().get_value(index)
             for index in self.pub_keys_view.selectedIndexes()
         ]
-        request_signature(selected_keys, self.app_data.config, self.threadpool, self)
+        request_signature(selected_keys, self.app_data.config, self)
 
     def _update_display(self, index: QtCore.QModelIndex):
         """Display key info summary in GUI text panel"""
@@ -412,89 +420,110 @@ class KeysTab(QtWidgets.QWidget):
 class KeyDownloadDialog(QtWidgets.QDialog):
     def __init__(self, *args, **kwargs):
         super().__init__(*args, **kwargs)
-        self.threadpool = QtCore.QThreadPool.globalInstance()
         self.setWindowTitle("Download public keys from keyserver")
         self.setWindowFlags(self.windowFlags() & ~QtCore.Qt.WindowContextHelpButtonHint)
 
         self.search_string = LineEdit()
         self.btn_search = QtWidgets.QPushButton("Search")
         self.btn_search.clicked.connect(self.search_keys)
+        self.btn_search.setEnabled(False)
+        self.search_string.textChanged.connect(
+            lambda text: self.btn_search.setEnabled(bool(text))
+        )
 
         self.key_list_view = QtWidgets.QListView()
         key_list_model = QtCore.QStringListModel()
+        self.key_list_view.setSelectionMode(
+            QtWidgets.QAbstractItemView.ExtendedSelection
+        )
         self.key_list_view.setModel(key_list_model)
 
-        self.btn_download = QtWidgets.QPushButton("Download")
+        self.btn_download = SelectionButton(
+            "Download", self.key_list_view.selectionModel()
+        )
         self.btn_download.clicked.connect(self.download_selected)
 
-        btn_cancel = QtWidgets.QPushButton("Close")
-        btn_cancel.clicked.connect(self.close)
+        self.label_results = QtWidgets.QLabel("")
+        self.set_number_of_keys_found(None)
 
-        layout = QtWidgets.QGridLayout()
-        layout.addWidget(
-            QtWidgets.QLabel(
-                "Enter a search term (e.g. user name, email, key fingerprint)"
-            ),
-            0,
-            0,
+        btn_box = QtWidgets.QDialogButtonBox(QtWidgets.QDialogButtonBox.Close)
+        btn_box.rejected.connect(self.reject)
+
+        self.setLayout(
+            grid_layout(
+                (
+                    QtWidgets.QLabel(
+                        "Enter a search term (e.g. user name, email, key fingerprint)"
+                    ),
+                ),
+                (self.search_string, self.btn_search),
+                (QtWidgets.QLabel("Select a key to download"),),
+                (self.key_list_view, self.btn_download),
+                (self.label_results,),
+                (None, btn_box),
+            )
         )
-        layout.addWidget(self.search_string, 1, 0)
-        layout.addWidget(self.btn_search, 1, 1)
-        layout.addWidget(QtWidgets.QLabel("Select a key to download"), 2, 0)
-        layout.addWidget(self.key_list_view, 3, 0)
-        layout.addWidget(self.btn_download, 3, 1)
-        layout.addWidget(btn_cancel, 4, 1)
-        self.setLayout(layout)
+
+    def set_number_of_keys_found(self, n: Optional[int]):
+        self.label_results.setText(f"Number of keys found: {'-' if n is None else n}")
 
     def search_keys(self):
         self.btn_search.setEnabled(False)
+        self.key_list_view.selectionModel().clear()
         self.key_list_view.model().setStringList([])
-        show_warning = warning_callback("GPG key search error")
-        worker = Worker(
-            crypt.search_keyserver,
-            self.search_string.text(),
-            self.parent().app_data.config.keyserver_url,
-            report_config=self.parent().app_data.config,
-            forward_errors=show_warning,
-        )
-        worker.signals.result.connect(
-            lambda keys: self.key_list_view.model().setStringList(
+        self.set_number_of_keys_found(None)
+
+        def on_result(keys):
+            self.key_list_view.model().setStringList(
                 [f"{k.uid} {k.fingerprint}" for k in keys]
             )
+            self.set_number_of_keys_found(len(keys))
+
+        run_thread(
+            crypt.search_keyserver,
+            f_kwargs=dict(
+                search_term=self.search_string.text(),
+                keyserver=self.parent().app_data.config.keyserver_url,
+            ),
+            report_config=self.parent().app_data.config,
+            forward_errors=warning_callback("GPG key search error"),
+            signals=dict(result=on_result),
         )
-        worker.signals.finished.connect(lambda: self.btn_search.setEnabled(True))
-        self.threadpool.start(worker)
 
     def download_selected(self):
         """Download keys selected in the key-search pop-up to local keyring"""
-        key_ids = []
-        for index in self.key_list_view.selectedIndexes():
-            key_ids.append(index.model().data(index).split()[-1])
-        if key_ids:
-            self.btn_download.setEnabled(False)
-            show_warning = warning_callback("GPG key search error")
-            worker = Worker(
-                crypt.download_keys,
-                key_ids,
-                self.parent().app_data.config.keyserver_url,
-                self.parent().app_data.config.gpg_store,
-                report_config=self.parent().app_data.config,
-                forward_errors=show_warning,
-            )
-            worker.signals.result.connect(
-                lambda: (
-                    self.parent().update_public_keys(),
-                    self.parent().update_display_selected_pub_key(),
-                )
-            )
-            worker.signals.finished.connect(lambda: self.btn_download.setEnabled(True))
-            self.threadpool.start(worker)
+        key_ids = [
+            index.model().data(index).split()[-1]
+            for index in self.key_list_view.selectedIndexes()
+        ]
+        if not key_ids:
+            return
+
+        def on_result():
+            self.parent().update_public_keys()
+            self.parent().update_display_selected_pub_key()
+
+        self.btn_download.setEnabled(False)
+        show_warning = warning_callback("GPG key search error")
+        run_thread(
+            crypt.download_keys,
+            f_kwargs=dict(
+                fingerprints=key_ids,
+                keyserver=self.parent().app_data.config.keyserver_url,
+                gpg_store=self.parent().app_data.config.gpg_store,
+            ),
+            report_config=self.parent().app_data.config,
+            forward_errors=show_warning,
+            signals=dict(
+                result=on_result,
+                finished=lambda: self.btn_download.setEnabled(True),
+            ),
+        )
 
 
 class KeyGenDialog(QtWidgets.QDialog):
     def __init__(self, *args, **kwargs):
         super().__init__(*args, **kwargs)
-        self.threadpool = QtCore.QThreadPool.globalInstance()
         self.setWindowTitle("Generate new key pair")
         self.setWindowFlags(self.windowFlags() & ~QtCore.Qt.WindowContextHelpButtonHint)
 
@@ -517,24 +546,29 @@ class KeyGenDialog(QtWidgets.QDialog):
         btn_show_pass.setCheckable(True)
         btn_show_pass.clicked.connect(self.toggle_password_visibility)
 
-        layout = QtWidgets.QGridLayout()
-        layout.addWidget(QtWidgets.QLabel("Full name"), 0, 0)
-        layout.addWidget(self.text_name_full, 0, 1)
-        layout.addWidget(QtWidgets.QLabel("(optional) institution/project"), 1, 0)
-        layout.addWidget(self.text_name_extra, 1, 1)
-        layout.addWidget(QtWidgets.QLabel("Institutional email"), 2, 0)
-        layout.addWidget(self.text_email, 2, 1)
-        layout.addWidget(QtWidgets.QLabel("Password"), 3, 0)
-        layout.addWidget(self.text_pass, 3, 1)
-        layout.addWidget(QtWidgets.QLabel("Password (repeat)"), 4, 0)
-        layout.addWidget(self.text_pass_repeat, 4, 1)
-        layout.addWidget(btn_show_pass, 4, 2)
-        layout.addWidget(btn_cancel, 5, 0)
-        layout.addWidget(self.btn_run, 5, 1)
-        layout.addWidget(
-            QtWidgets.QLabel("Key generation can take a few minutes"), 6, 0, 1, 3
+        self.setLayout(
+            grid_layout(
+                (QtWidgets.QLabel("Full name"), self.text_name_full),
+                (
+                    QtWidgets.QLabel("(optional) institution/project"),
+                    self.text_name_extra,
+                ),
+                (QtWidgets.QLabel("Institutional email"), self.text_email),
+                (QtWidgets.QLabel("Password"), self.text_pass),
+                (
+                    QtWidgets.QLabel("Password (repeat)"),
+                    self.text_pass_repeat,
+                    btn_show_pass,
+                ),
+                (btn_cancel, self.btn_run),
+                (
+                    GridLayoutCell(
+                        QtWidgets.QLabel("Key generation can take a few minutes"),
+                        span=3,
+                    ),
+                ),
+            )
         )
-        self.setLayout(layout)
 
     def toggle_password_visibility(self, show: bool):
         mode = QtWidgets.QLineEdit.Normal if show else QtWidgets.QLineEdit.Password
@@ -586,9 +620,7 @@ class KeyGenDialog(QtWidgets.QDialog):
                 self.parent().app_data.config.key_authority_fingerprint
                 and not self.parent().app_data.config.offline
             ):
-                request_signature(
-                    (key,), self.parent().app_data.config, self.threadpool, self
-                )
+                request_signature((key,), self.parent().app_data.config, self)
             self.clear_form()
             self.parent().update_private_keys()
             self.parent().update_public_keys()
@@ -604,19 +636,22 @@ class KeyGenDialog(QtWidgets.QDialog):
                 name_extra = f"({name_extra})"
             name_full = name_full + " " + name_extra
 
-        worker = Worker(
+        run_thread(
             crypt.create_key,
-            name_full,
-            self.text_email.text(),
-            self.text_pass.text(),
-            self.text_pass_repeat.text(),
-            self.parent().app_data.config.gpg_store,
+            f_kwargs=dict(
+                full_name=name_full,
+                email=self.text_email.text(),
+                pwd=self.text_pass.text(),
+                pwd_repeat=self.text_pass_repeat.text(),
+                gpg_store=self.parent().app_data.config.gpg_store,
+            ),
             forward_errors=show_warning,
             report_config=self.parent().app_data.config,
+            signals=dict(
+                result=self.post_key_creation,
+                finished=lambda: self.btn_run.setEnabled(True),
+            ),
         )
-        worker.signals.result.connect(self.post_key_creation)
-        worker.signals.finished.connect(lambda: self.btn_run.setEnabled(True))
-        self.threadpool.start(worker)
 
 
 def click_show_details(msgbox: QtWidgets.QMessageBox):
@@ -628,7 +663,6 @@ def click_show_details(msgbox: QtWidgets.QMessageBox):
 def request_signature(
     keys: Sequence[gpg.Key],
     config: Config,
-    threadpool: QtCore.QThreadPool,
     parent: Optional[QtWidgets.QWidget] = None,
 ):
     """Request the specified keys to be signed by the key validation authority.
@@ -652,16 +686,17 @@ def request_signature(
             click_show_details(msgbox=msg)
         status = open_window(msg)
         if status == QtWidgets.QMessageBox.Yes:
-            worker = Worker(
+            run_thread(
                 request_sigs_workflow.request_sigs,
-                key_ids=[key.key_id],
-                config=config,
+                f_kwargs=dict(
+                    key_ids=[key.key_id],
+                    config=config,
+                ),
                 capture_loggers=(request_sigs_workflow.logger,),
                 forward_errors=show_warning,
                 report_config=config,
+                signals=dict(result=lambda _: show_ok()),
             )
-            worker.signals.result.connect(lambda _: show_ok())
-            threadpool.start(worker)
 
 
 def ok_message(title: str, msg: str, parent=None) -> Callable:
