@@ -15,9 +15,9 @@ from .decrypt_tab import DecryptTab
 from .encrypt_tab import EncryptTab
 from .keys_tab import KeysTab
 from .model import AppData
-from .parallel import Worker
+from .parallel import run_thread
 from .pyside import QtCore, QtGui, QtWidgets, QAction, open_window
-from .settings_tab import SettingsTab, config_to_settings
+from .settings_tab import SettingsTab
 from .transfer_tab import TransferTab
 from ..core.versioncheck import check_version
 from ..utils.config import load_config, save_config, config_to_dict
@@ -106,9 +106,7 @@ class MainWindow(QtWidgets.QMainWindow):
         menu_help.addAction(action_about)
 
     def closeEvent(self, event):
-        if config_to_settings(self.app_data.config) != config_to_settings(
-            self.config_on_disk
-        ):
+        if self.app_data.config != self.config_on_disk:
             reply = QtWidgets.QMessageBox.question(
                 self,
                 "Persist changed settings?",
@@ -142,12 +140,11 @@ class MainWindow(QtWidgets.QMainWindow):
                 check_version(self.app_data.config.repo_url, gui_formatting=True)
                 return "\n".join(format(warning.message) for warning in w)
 
-        worker = Worker(
+        run_thread(
             get_warnings,
-            report_config=self.app_data.config,
+            f_kwargs=dict(report_config=self.app_data.config),
+            signals=dict(result=lambda x: x and show_warning(self.title, x, self)),
         )
-        worker.signals.result.connect(lambda x: x and show_warning(self.title, x, self))
-        QtCore.QThreadPool.globalInstance().start(worker)
 
     def show_about(self):
         msg = QtWidgets.QMessageBox(parent=self)
