@@ -1,9 +1,7 @@
-from collections import defaultdict
 from dataclasses import dataclass, field
 from pathlib import Path
 from typing import (
     Any,
-    Dict,
     Hashable,
     Iterable,
     List,
@@ -11,14 +9,16 @@ from typing import (
     Tuple,
     Union,
     Sequence,
-    Callable,
+    Type,
 )
 
 
 from .pyside import QtCore
+from .ui_model_bind import ClassWithListener, DictWithListener
 from ..core.crypt import gpg
 from ..core.metadata import Purpose
 from ..utils.config import Config
+from ..protocols import Protocol, protocols, sftp
 
 
 class TableModel(QtCore.QAbstractTableModel):
@@ -113,8 +113,12 @@ class KeyValueListModel(QtCore.QAbstractListModel):
         raise TypeError("Wrong index type for index.")
 
 
+def protocol_factory():
+    return {p: DictWithListener() for p in protocols}
+
+
 @dataclass
-class AppData:
+class AppData(ClassWithListener):
     """Settings that are specific to the sett GUI"""
 
     config: Config
@@ -135,8 +139,8 @@ class AppData:
     decrypt_output_location: Path = field(default_factory=Path.home)
     decrypt_files: List[str] = field(default_factory=list)
 
-    transfer_protocol_name: str = "sftp"
-    transfer_protocol_args: Dict = field(default_factory=dict)
+    transfer_protocol_type: Type[Protocol] = sftp.Protocol
+    transfer_protocol_args: dict = field(default_factory=protocol_factory)
     transfer_files: List[str] = field(default_factory=list)
     transfer_verify_dtr: bool = True
 
@@ -146,24 +150,8 @@ class AppData:
 
     validation_authority_key: Optional[gpg.Key] = None
 
-    def __setattr__(self, attr, value):
-        # Each time an attribute is set, check whether it is being watched,
-        # and if so, run the associated functions.
-        super().__setattr__(attr, value)
-        if hasattr(self, "_listeners"):
-            for callback in self._listeners[attr]:
-                callback()
-
-    def add_listener(self, attr, callback: Callable):
-        """Add a listener to the specified attribute. Each time the value of
-        the attributed will change, the specified callback function(s) will
-        be called.
-        """
-        self._listeners[attr].append(callback)
-
     def __post_init__(self):
-        # pylint: disable=attribute-defined-outside-init
-        self._listeners = defaultdict(list)
+        super().__init__()
         if self.config.offline:
             self.encrypt_verify_dtr = self.transfer_verify_dtr = False
         if self.config.output_dir:
