@@ -3,30 +3,36 @@ import platform
 import sys
 import traceback
 import warnings
-from typing import Sequence, Callable, Optional, cast
 from contextlib import contextmanager
+from typing import Sequence, Callable, Optional, Dict, cast
 
-from .pyside import QtCore
-from ..utils.log import EmojiLevelFormatter, LOG_FORMAT, LOG_FORMAT_WITH_EMOJI, LogStyle
+from .pyside import QtCore, get_application_global_instance
+from ..core.secret import enforce_secret_by_signature
+from ..utils.config import Config
 from ..utils.error_handling import (
     suppress_exceptions,
     log_exceptions,
     error_report_on_exception,
 )
-from ..utils.config import Config
-from ..core.secret import enforce_secret_by_signature
-
-from .. import RUNTIME_INFO
+from ..utils.log import EmojiLevelFormatter, LOG_FORMAT, LOG_FORMAT_WITH_EMOJI
 
 
 class LoggerHandler(logging.Handler):
     def __init__(self, level=logging.NOTSET, callback=None):
         super().__init__(level)
         self.callback = callback
+        palette = get_application_global_instance().palette()
+        self.log_style = {
+            logging.DEBUG: "gray",
+            logging.INFO: palette.color(palette.ColorRole.Text),
+            logging.WARNING: "orange",
+            logging.ERROR: "red",
+            logging.CRITICAL: "red",
+        }
 
     def emit(self, record: logging.LogRecord):
         if self.callback:
-            color = LogStyle.HTML.get_style(record.levelno)
+            color = self.log_style.get(record.levelno)
             self.callback(
                 f'<span style="color: {color}">{self.format(record)}</span>'
                 if color
@@ -35,7 +41,6 @@ class LoggerHandler(logging.Handler):
 
 
 class WorkerSignals(QtCore.QObject):
-    started = QtCore.Signal()
     finished = QtCore.Signal()
     error = QtCore.Signal(tuple)
     result = QtCore.Signal(object)
@@ -101,8 +106,6 @@ class Worker(QtCore.QRunnable):
             self.capture_loggers, self.signals
         ), warnings.catch_warnings(record=True) as user_warnings:
             try:
-                self.signals.started.emit()
-                self.signals.logging.emit(RUNTIME_INFO)
                 result = self.fn(*self.args, **self.kwargs)
             except Exception:  # pylint: disable=broad-except
                 exctype, value, _ = sys.exc_info()
@@ -118,7 +121,7 @@ class Worker(QtCore.QRunnable):
 
 
 @contextmanager
-def attach_loggers_to_signals(loggers, signals):
+def attach_loggers_to_signals(loggers: Sequence[logging.Logger], signals):
     logger_handler = LoggerHandler(callback=signals.logging.emit)
     logger_handler.setFormatter(
         logging.Formatter(LOG_FORMAT)
@@ -131,3 +134,18 @@ def attach_loggers_to_signals(loggers, signals):
     yield None
     for logger in loggers:
         logger.removeHandler(logger_handler)
+
+
+def run_thread(
+    f: Callable,
+    f_kwargs,
+    signals: Optional[Dict[str, Callable]] = None,
+    **worker_kwargs,
+):
+    pool = QtCore.QThreadPool.globalInstance()
+    worker = Worker(f, **f_kwargs, **worker_kwargs)
+    if signals is None:
+        signals = {}
+    for sig_name, sig_callback in signals.items():
+        getattr(worker.signals, sig_name).connect(sig_callback)
+    pool.start(worker)
