@@ -7,6 +7,9 @@ try:
     def open_window(window: QtWidgets.QWidget):
         return window.exec()
 
+    def get_application_global_instance():
+        return QtWidgets.QApplication.instance()
+
 
 except ImportError:
     from PySide2 import QtCore, QtGui, QtWidgets  # type: ignore
@@ -14,3 +17,6 @@ except ImportError:
 
     def open_window(window: QtWidgets.QWidget):
         return window.exec_()
+
+    def get_application_global_instance():
+        return QtWidgets.QApplication.instance()
