@@ -1,5 +1,4 @@
 import enum
-import operator
 import time
 from functools import partial
 from typing import Iterator, Optional, Sequence
@@ -11,16 +10,19 @@ from .component import (
     get_text_input,
     MandatoryLabel,
     create_verify_dtr_checkbox,
-    FieldMapping,
     ToolBar,
     LineEdit,
-    Action,
+    grid_layout,
+    vbox_layout,
+    hbox_layout,
 )
 from .file_selection_widget import ArchiveOnlyFileSelectionWidget
-from .parallel import Worker
 from .pyside import QtCore, QtGui, QtWidgets, QAction, open_window
+from .ui_model_bind import bind, TextControl, OptionalTextControl, PathControl
 from .. import protocols
 from ..core.error import UserError
+from ..protocols.liquid_files import Protocol as liquid_files
+from ..protocols.sftp import Protocol as sftp
 from ..utils.config import Connection, ConnectionStore
 from ..workflows import transfer
 
@@ -28,17 +30,15 @@ from ..workflows import transfer
 class TransferTab(QtWidgets.QWidget, TabMixin):
     def __init__(self, *args, **kwargs):
         super().__init__(*args, **kwargs)
-        self.threadpool = QtCore.QThreadPool.globalInstance()
         self.app_data = self.parent().app_data
-        self.app_data.transfer_protocol_args = {"sftp": {}, "liquid_files": {}}
         self.connections_model = QtCore.QStringListModel()
 
         files_panel = self.create_files_panel()
 
         verify_dtr_group = QtWidgets.QGroupBox()
-        verify_dtr_layout = QtWidgets.QVBoxLayout(verify_dtr_group)
-        verify_dtr_layout.addWidget(
-            create_verify_dtr_checkbox(self.app_data, "transfer_verify_dtr")
+        vbox_layout(
+            create_verify_dtr_checkbox(self.app_data, "transfer_verify_dtr"),
+            parent=verify_dtr_group,
         )
 
         options_panel = self.create_options_panel()
@@ -47,23 +47,22 @@ class TransferTab(QtWidgets.QWidget, TabMixin):
         self.create_console()
         self.create_progress_bar()
 
-        layout = QtWidgets.QVBoxLayout(self)
-        for widget in (
+        vbox_layout(
             files_panel,
             verify_dtr_group,
             options_panel,
             self.run_panel,
             self.console,
             self.progress_bar,
-        ):
-            layout.addWidget(widget)
+            parent=self,
+        )
 
     def _enable_buttons(self):
         self.set_buttons_enabled(len(self.app_data.transfer_files) > 0)
 
     def create_files_panel(self):
         box = ArchiveOnlyFileSelectionWidget(
-            "Encrypted files to transfer (hint: you can drag & drop files)", self
+            title="Encrypted files to transfer", parent=self
         )
         box.file_list_model.layoutChanged.connect(
             lambda: setattr(self.app_data, "transfer_files", box.get_list())
@@ -71,114 +70,70 @@ class TransferTab(QtWidgets.QWidget, TabMixin):
         return box
 
     def create_sftp_options_panel(self):
-        field_mapping = FieldMapping()
-
-        text_username = field_mapping.bind_text("username", LineEdit())
+        sftp_args = self.app_data.transfer_protocol_args[sftp]
+        text_username = LineEdit()
         text_username.setStatusTip("Username on the SFTP server")
-        text_destination_dir = field_mapping.bind_text("destination_dir", LineEdit())
-        text_username.textChanged.connect(
-            lambda: operator.setitem(
-                self.app_data.transfer_protocol_args["sftp"],
-                "username",
-                text_username.text(),
-            )
-        )
-        text_host = field_mapping.bind_text("host", LineEdit())
-        text_host.setStatusTip("URL of the SFTP server with an optional port " "number")
-        text_host.textChanged.connect(
-            lambda: operator.setitem(
-                self.app_data.transfer_protocol_args["sftp"], "host", text_host.text()
-            )
-        )
-        text_jumphost = field_mapping.bind_text("jumphost", LineEdit())
-        text_jumphost.setStatusTip("(optional) URL of the jumphost server")
-        text_jumphost.textChanged.connect(
-            lambda: operator.setitem(
-                self.app_data.transfer_protocol_args["sftp"],
-                "jumphost",
-                text_jumphost.text() or None,
-            )
-        )
-        text_destination_dir.textChanged.connect(
-            lambda: operator.setitem(
-                self.app_data.transfer_protocol_args["sftp"],
-                "destination_dir",
-                text_destination_dir.text(),
-            )
-        )
+        bind(sftp_args, "username", text_username, TextControl)
+
+        text_destination_dir = LineEdit()
         text_destination_dir.setStatusTip(
             "Relative or absolute path to the "
             "destination directory on the SFTP "
             "server"
         )
-        pkey_location = field_mapping.bind_path(
-            "pkey", PathInput(directory=False, path=None)
-        )
+        bind(sftp_args, "destination_dir", text_destination_dir, TextControl)
+
+        text_host = LineEdit()
+        text_host.setStatusTip("URL of the SFTP server with an optional port " "number")
+        bind(sftp_args, "host", text_host, TextControl)
+
+        text_jumphost = LineEdit()
+        text_jumphost.setStatusTip("(optional) URL of the jumphost server")
+        bind(sftp_args, "jumphost", text_jumphost, OptionalTextControl)
+
+        pkey_location = PathInput(directory=False, path=None)
         pkey_location.status_tip = "Path to the private SSH key used for authentication"
-        pkey_location.on_path_change(
-            lambda path: operator.setitem(
-                self.app_data.transfer_protocol_args["sftp"],
-                "pkey",
-                "" if path is None else str(path),
-            )
-        )
-        pkey_password = field_mapping.bind_text("pkey_password", LineEdit())
+        bind(sftp_args, "pkey", pkey_location, PathControl)
+
+        pkey_password = LineEdit()
         pkey_password.setStatusTip("Passphrase for the SSH private key")
         pkey_password.setEchoMode(QtWidgets.QLineEdit.Password)
-        pkey_password.textChanged.connect(
-            lambda: operator.setitem(
-                self.app_data.transfer_protocol_args["sftp"],
-                "pkey_password",
-                pkey_password.text(),
-            )
-        )
+        bind(sftp_args, "pkey_password", pkey_password, TextControl)
 
         box = QtWidgets.QGroupBox()
         box.setFlat(True)
-        box.load_connection = field_mapping.load
-        layout = QtWidgets.QGridLayout(box)
-        layout.addWidget(MandatoryLabel("User name"), 0, 0)
-        layout.addWidget(text_username, 0, 1)
-        layout.addWidget(MandatoryLabel("Host URL"), 1, 0)
-        layout.addWidget(text_host, 1, 1)
-        layout.addWidget(QtWidgets.QLabel("Jumphost URL"), 2, 0)
-        layout.addWidget(text_jumphost, 2, 1)
-        layout.addWidget(MandatoryLabel("Destination directory"), 3, 0)
-        layout.addWidget(text_destination_dir, 3, 1)
-        layout.addWidget(QtWidgets.QLabel("SSH key location"), 4, 0)
-        layout.addWidget(pkey_location.text, 4, 1)
-        layout.addWidget(pkey_location.btn, 4, 2)
-        layout.addWidget(pkey_location.btn_clear, 4, 3)
-        layout.addWidget(QtWidgets.QLabel("SSH key password"), 5, 0)
-        layout.addWidget(pkey_password, 5, 1)
+        grid_layout(
+            (MandatoryLabel("User name"), text_username),
+            (MandatoryLabel("Host URL"), text_host),
+            (QtWidgets.QLabel("Jumphost URL"), text_jumphost),
+            (MandatoryLabel("Destination directory"), text_destination_dir),
+            (
+                QtWidgets.QLabel("SSH key location"),
+                pkey_location.text,
+                pkey_location.btn,
+                pkey_location.btn_clear,
+            ),
+            (QtWidgets.QLabel("SSH key password"), pkey_password),
+            parent=box,
+        )
         return box
 
     def create_liquidfiles_options_panel(self):
-        field_mapping = FieldMapping()
-        text_host = field_mapping.bind_text("host", LineEdit())
-        text_api_key = field_mapping.bind_text("api_key", LineEdit())
-        text_host.textChanged.connect(
-            lambda: operator.setitem(
-                self.app_data.transfer_protocol_args["liquid_files"],
-                "host",
-                text_host.text(),
-            )
-        )
-        text_api_key.textChanged.connect(
-            lambda: operator.setitem(
-                self.app_data.transfer_protocol_args["liquid_files"],
-                "api_key",
-                text_api_key.text(),
-            )
-        )
+        lf_args = self.app_data.transfer_protocol_args[liquid_files]
+
+        text_host = LineEdit()
+        bind(lf_args, "host", text_host, TextControl)
+
+        text_api_key = LineEdit()
+        bind(lf_args, "api_key", text_api_key, TextControl)
+
         box = QtWidgets.QGroupBox()
         box.setFlat(True)
-        box.load_connection = field_mapping.load
-        layout = QtWidgets.QGridLayout(box)
-        layout.addWidget(MandatoryLabel("Host URL"), 0, 0)
-        layout.addWidget(text_host, 0, 1)
-        layout.addWidget(MandatoryLabel("API Key"), 1, 0)
-        layout.addWidget(text_api_key, 1, 1)
+        grid_layout(
+            (MandatoryLabel("Host URL"), text_host),
+            (MandatoryLabel("API Key"), text_api_key),
+            parent=box,
+        )
         return box
 
     def create_conn_actions(
@@ -252,13 +207,17 @@ class TransferTab(QtWidgets.QWidget, TabMixin):
         def save():
             label = connections_selection.currentText()
             if confirm(f"Do you want to save '{label}' profile?"):
-                protocol_name = self.app_data.transfer_protocol_name
+                protocol_name = protocols.protocol_name[
+                    self.app_data.transfer_protocol_type
+                ]
                 connection = Connection(
                     protocol=protocol_name,
                     parameters=self.app_data.transfer_protocol_args[
-                        protocol_name
+                        self.app_data.transfer_protocol_type
                     ].copy(),
                 )
+                if "pkey_password" in connection.parameters:
+                    del connection.parameters["pkey_password"]
                 self.app_data.config.connections[label] = connection
                 connection_store.save(label, connection)
 
@@ -271,7 +230,7 @@ class TransferTab(QtWidgets.QWidget, TabMixin):
             ("Delete the current connection profile", delete, True, "trash-2"),
             ("Save the current connection profile", save, True, "save"),
         ):
-            action = Action(QtGui.QIcon(f":icon/feather/{icon}.png"), tip, self)
+            action = QAction(QtGui.QIcon(f":icon/feather/{icon}.png"), tip, self)
             action.triggered.connect(fn)
             if needs_selection:
                 connections_selection.currentTextChanged.connect(
@@ -309,16 +268,17 @@ class TransferTab(QtWidgets.QWidget, TabMixin):
             if not connection:
                 return
             get_btn_from_group(protocol_btn_grp, connection.protocol).click()
-            protocol_boxes[connection.protocol].load_connection(connection.parameters)
             self.app_data.transfer_protocol_args[
-                self.app_data.transfer_protocol_name
+                self.app_data.transfer_protocol_type
             ].update(connection.parameters)
 
         connections_selection.currentTextChanged.connect(load_connection)
 
         def toggle_protocol(btn, state: bool):
             if state:
-                self.app_data.transfer_protocol_name = btn.text()
+                self.app_data.transfer_protocol_type = protocols.parse_protocol(
+                    btn.text()
+                )
             protocol_boxes[btn.text()].setVisible(state)
 
         protocol_btn_grp.buttonToggled.connect(toggle_protocol)
@@ -326,29 +286,15 @@ class TransferTab(QtWidgets.QWidget, TabMixin):
         toolbar = ToolBar("Options", self)
         for action in self.create_conn_actions(connections_selection):
             toolbar.addAction(action)
-        layout_connection = QtWidgets.QHBoxLayout()
-        layout_connection.addWidget(connections_selection)
-        layout_connection.addWidget(toolbar)
+        layout_connection = hbox_layout(connections_selection, toolbar)
+        layout_protocol_buttons = hbox_layout(*protocol_btn_grp.buttons())
 
-        layout_protocol_buttons = QtWidgets.QHBoxLayout()
-        for btn in protocol_btn_grp.buttons():
-            layout_protocol_buttons.addWidget(btn)
-
-        layout_protocol = QtWidgets.QVBoxLayout()
-        layout_protocol.addLayout(layout_protocol_buttons)
-        for protocol_box in protocol_boxes.values():
-            layout_protocol.addWidget(protocol_box)
-
-        layout = QtWidgets.QVBoxLayout(box)
-        layout.addLayout(layout_connection)
-        layout.addLayout(layout_protocol)
+        layout_protocol = vbox_layout(layout_protocol_buttons, *protocol_boxes.values())
+        vbox_layout(layout_connection, layout_protocol, parent=box)
         load_connection()
         return box
 
     def transfer(self, dry_run=False):
-        progress = GuiProgress()
-        progress.updated.connect(self.progress_bar.setValue)
-
         second_factor = None
 
         class Msg(enum.Enum):
@@ -379,31 +325,28 @@ class TransferTab(QtWidgets.QWidget, TabMixin):
         specific_protocol_args = {
             protocols.sftp.Protocol: {"two_factor_callback": second_factor_callback}
         }
-        protocol_type = protocols.protocols_by_name[
-            self.app_data.transfer_protocol_name
-        ]
-        protocol = protocol_type(
+        protocol = self.app_data.transfer_protocol_type(
             **self.app_data.transfer_protocol_args[
-                self.app_data.transfer_protocol_name
+                self.app_data.transfer_protocol_type
             ],
-            **specific_protocol_args.get(protocol_type, {}),
+            **specific_protocol_args.get(self.app_data.transfer_protocol_type, {}),
         )
 
-        worker = Worker(
+        msg_signal.msg.connect(show_second_factor_dialog)
+        self.run_workflow_thread(
             transfer.transfer,
-            files=self.app_data.transfer_files,
+            f_kwargs=dict(
+                files=self.app_data.transfer_files,
+                protocol=protocol,
+                config=self.app_data.config,
+                dry_run=dry_run,
+                verify_dtr=self.app_data.transfer_verify_dtr,
+                progress=GuiProgress(self.progress_bar.setValue),
+            ),
             capture_loggers=(transfer.logger, protocols.sftp.logger),
-            protocol=protocol,
-            config=self.app_data.config,
-            dry_run=dry_run,
-            verify_dtr=self.app_data.transfer_verify_dtr,
-            progress=progress,
             ignore_exceptions=True,
             report_config=self.app_data.config,
         )
-        self.add_worker_actions(worker)
-        msg_signal.msg.connect(show_second_factor_dialog)
-        self.threadpool.start(worker)
 
 
 class ConnectionDialog(QtWidgets.QDialog):
@@ -439,11 +382,7 @@ class ConnectionDialog(QtWidgets.QDialog):
         set_ok_enabled(self.text_field.text())
         self.text_field.textChanged.connect(set_ok_enabled)
 
-        layout = QtWidgets.QVBoxLayout()
-        layout.addWidget(QtWidgets.QLabel(label))
-        layout.addWidget(self.text_field)
-        layout.addWidget(btn_box)
-        self.setLayout(layout)
+        self.setLayout(vbox_layout(QtWidgets.QLabel(label), self.text_field, btn_box))
 
 
 def get_btn_from_group(btn_grp: QtWidgets.QButtonGroup, text: str):
