@@ -1,15 +1,14 @@
-import inspect
-from typing import Callable, Type, Dict, Tuple
+from typing import Type, Dict, Tuple
 
 from .protocol import Protocol
 from . import liquid_files, sftp
-from ..utils.progress import ProgressInterface
 
 protocols: Tuple[Type[Protocol], ...] = (sftp.Protocol, liquid_files.Protocol)
 protocols_by_name: Dict[str, Type[Protocol]] = {
     p.__module__.replace(__name__ + ".", ""): p for p in protocols
 }
 __all__ = tuple(protocols_by_name)
+protocol_name = {protocol: name for name, protocol in protocols_by_name.items()}
 
 
 def parse_protocol(s: str) -> Type[Protocol]:
