@@ -5,7 +5,7 @@ from typing import Tuple
 
 def encode(name: str, filename: str, data: bytes, extra_data=()) -> Tuple[str, bytes]:
     boundary = uuid.uuid4().hex
-    content_type = "multipart/form-data; boundary={}".format(boundary)
+    content_type = f"multipart/form-data; boundary={boundary}"
     boundary_b = boundary.encode()
     inner_content_type = (
         mimetypes.guess_type(filename)[0] or "application/octet-stream"
