@@ -60,7 +60,7 @@ def write_error_report(config: Config, log: str, context: Optional[str] = None)
     """Writes a error report containing sys.argv, config and a log.
     Returns the filename of the report."""
     f_name = f"error_report-{datetime.now().strftime('%Y-%m-%dT%H-%M-%S')}.txt"
-    with open(os.path.join(config.log_dir, f_name), "w") as f:
+    with open(os.path.join(config.log_dir, f_name), "w", encoding="utf-8") as f:
         f.write("cmd: " + repr(sys.argv) + "\n")
         if context is not None:
             f.write("context: " + context + "\n")
