@@ -1,4 +1,3 @@
-import enum
 import io
 import logging
 import logging.handlers
@@ -21,39 +20,17 @@ LOG_FORMAT_WITH_TIME_AND_PID = "[%(asctime)s] [PID:%(process)d] " + LOG_FORMAT
 colorama.init()
 
 
-class LogStyle(enum.Enum):
-    EMOJI = enum.auto()
-    TERMINAL = enum.auto()
-    HTML = enum.auto()
-
-    def get_style(self, log_level: int) -> Optional[str]:
-        style = {
-            LogStyle.EMOJI: {
-                logging.DEBUG: "🔬",
-                logging.INFO: "ℹ️",
-                logging.WARNING: "⚠️",
-                logging.ERROR: "💥",
-                logging.CRITICAL: "☢️",
-            },
-            LogStyle.TERMINAL: {
-                logging.WARNING: colorama.Fore.YELLOW,
-                logging.ERROR: colorama.Fore.RED,
-                logging.CRITICAL: colorama.Fore.RED,
-            },
-            LogStyle.HTML: {
-                logging.DEBUG: "gray",
-                logging.INFO: "black",
-                logging.WARNING: "orange",
-                logging.ERROR: "red",
-                logging.CRITICAL: "red",
-            },
-        }
-        return style[self].get(log_level)
-
-
 class EmojiLevelFormatter(logging.Formatter):
+    log_style = {
+        logging.DEBUG: "🔬",
+        logging.INFO: "ℹ️",
+        logging.WARNING: "⚠️",
+        logging.ERROR: "💥",
+        logging.CRITICAL: "☢️",
+    }
+
     def format(self, record: logging.LogRecord):
-        emoji = LogStyle.EMOJI.get_style(record.levelno)
+        emoji = self.log_style.get(record.levelno)
         if emoji:
             # Assigning to `levelname` changes the value also for other
             # formatters so it's necessary to use a copy of the LogRecord
@@ -65,8 +42,14 @@ class EmojiLevelFormatter(logging.Formatter):
 
 
 class ColoredTerminalFormatter(logging.Formatter):
+    log_style = {
+        logging.WARNING: colorama.Fore.YELLOW,
+        logging.ERROR: colorama.Fore.RED,
+        logging.CRITICAL: colorama.Fore.RED,
+    }
+
     def format(self, record: logging.LogRecord) -> str:
-        color = LogStyle.TERMINAL.get_style(record.levelno)
+        color = self.log_style.get(record.levelno)
         if color:
             return f"{color}{super().format(record)}{colorama.Fore.RESET}"
         return super().format(record)
@@ -99,7 +82,7 @@ def log_task(
     ok_logger("OK")
 
 
-def log_timing(logger):
+def log_timing(logger: logging.Logger):
     """A function decorator, logging processing time taken by the contained code"""
 
     def decorator(f):
@@ -127,12 +110,23 @@ def sec_to_hours(seconds: int) -> str:
 
 
 def log_runtime_info(logger: logging.Logger):
-    """Logs:
+    """Logs version of the application, dependencies, and platform.
+
     - application version
     - versions of the main dependencies
+    - python version
     - version of the operating system where the application is running
     """
-    logger.info(RUNTIME_INFO)
+
+    def decorator(f):
+        @wraps(f)
+        def wrapped_f(*args, **kwargs):
+            logger.info(RUNTIME_INFO)
+            return f(*args, **kwargs)
+
+        return wrapped_f
+
+    return decorator
 
 
 class LoggerExt(logging.Logger):
