@@ -1,15 +1,12 @@
 import os
-import json
 
 from functools import partial
 from typing import List, Optional
 
-from ..core.error import UserError
 from ..core.archive import (
     check_package,
     extract,
     unpack_from_stream,
-    METADATA_FILE,
     DATA_ARCHIVE,
     DATA_FILE_ENCRYPTED,
     CHECKSUM_FILE,
@@ -32,14 +29,9 @@ from ..core.filesystem import (
     unique_filename,
     check_space,
 )
-from ..core.checksum import (
-    verify_checksums,
-    compute_checksum_sha256,
-    read_checksum_file,
-)
-from ..core.metadata import MetaData
+from ..core.checksum import verify_checksums, read_checksum_file
 from ..core.secret import Secret
-from ..utils.log import create_logger, log_timing
+from ..utils.log import create_logger, log_runtime_info, log_timing
 from ..utils.progress import ProgressInterface, subprogress
 from ..utils.config import Config
 
@@ -48,6 +40,7 @@ logger = create_logger(__name__)
 
 
 @log_timing(logger)
+@log_runtime_info(logger)
 def decrypt(
     files: List[str],
     *,
@@ -139,21 +132,9 @@ def decrypt_archive(
                 gpg_store=config.gpg_store,
             )
 
-        with logger.log_task("Verifying checksums..."), subprogress(
-            progress, step_completion_increase=0.05
-        ) as scaled_progress, extract_with_progress(
-            archive_file, scaled_progress, METADATA_FILE, DATA_FILE_ENCRYPTED
-        ) as (
-            f_metadata,
-            f_data,
-        ):
-            metadata = MetaData.from_dict(json.load(f_metadata))
-            if metadata.checksum.lower() != compute_checksum_sha256(f_data):
-                raise UserError(f"Checksum mismatch for {f_data.name}")
-
-        unpacked_files: List[str] = list()
+        unpacked_files: List[str] = []
         with logger.log_task("Decrypting data..."), subprogress(
-            progress, step_completion_increase=0.9
+            progress, step_completion_increase=0.95
         ) as scaled_progress, extract_with_progress(
             archive_file, scaled_progress, DATA_FILE_ENCRYPTED
         ) as f_data:
