@@ -8,7 +8,7 @@ from typing import List, Optional, Callable, Tuple, Union, Iterable
 from zipfile import ZipFile, ZipInfo, ZIP_STORED
 
 from ..utils.config import Config
-from ..utils.log import create_logger, log_timing
+from ..utils.log import create_logger, log_runtime_info, log_timing
 from ..core.archive import (
     write_tar,
     ArchiveInMemoryFile,
@@ -118,6 +118,7 @@ check_compression_level = check_integer_in_range(min_value=0, max_value=9)
 
 
 @log_timing(logger)
+@log_runtime_info(logger)
 def encrypt(  # pylint: disable=too-many-statements
     files: List[str],
     *,
@@ -159,6 +160,10 @@ def encrypt(  # pylint: disable=too-many-statements
     files_to_encrypt = list(search_files_recursively(files))
 
     with logger.log_task("Input data check"):
+        if not files_to_encrypt:
+            raise UserError(
+                "No input files found. Did you try encrypting an empty directory?"
+            )
         check_file_read_permission(files_to_encrypt)
         # Retrieve the lowest common directory of all input files/directories.
         root_dir = os.path.commonpath(
