@@ -3,13 +3,14 @@ from typing import Sequence
 from .upload_keys import upload_keys
 from ..core.crypt import verify_key_length, search_pub_key
 from ..core.error import UserError
-from ..utils.log import create_logger
+from ..utils.log import create_logger, log_runtime_info
 from ..utils.config import Config
 
 
 logger = create_logger(__name__)
 
 
+@log_runtime_info(logger)
 def request_sigs(key_ids: Sequence[str], *, config=Config):
     """Requests signatures"""
     if config.offline:
