@@ -6,13 +6,14 @@ from ..core.crypt import (
     search_pub_key,
 )
 from ..core.error import UserError
-from ..utils.log import create_logger
+from ..utils.log import create_logger, log_runtime_info
 from ..utils.config import Config
 
 
 logger = create_logger(__name__)
 
 
+@log_runtime_info(logger)
 def upload_keys(key_ids: Sequence[str], *, config: Config):
     """Upload keys"""
     if config.offline:
